@@ -38,7 +38,7 @@ use CGI::FormBuilder;
 use HTML::Entities;
 use URI::Escape;
 
-our $VERSION = '0.98';
+our $VERSION = '0.99';
 
 =encoding utf8
 
@@ -191,6 +191,24 @@ The name of the database table.
 Specify which column in the table is the primary key.  If not given, defaults to
 id.
 
+=item C<where_filter> (optional)
+
+Specify one or more 'where' clauses to use to filter the table. For example:
+
+    simple_crud(
+        prefix => 'bar',
+        where_filter => {user_id => 1000},
+        ...
+    );
+
+This would cause only rows with an user_id of 1000 to be displayed.
+
+The C<where_filter> parameter takes a hashref describing the WHERE clause, as 
+used by L<Dancer::Plugin::Database>'s C<quick_select> convenience method for
+example - see the 
+L<where clause documentation in Dancer::Plugin::Database::Core::Handle|Dancer::Plugin::Database::Core::Handle/WHERE-clauses-as-hashrefs>.
+
+
 =item C<db_connection_name> (optional)
 
 We use L<Dancer::Plugin::Database> to obtain database connections.  This option
@@ -981,6 +999,7 @@ SEARCHFORM
         ? ", $table_name.$key_column AS actions"
         : '';
     my $query = "SELECT $col_list $add_actions FROM $table_name";
+    my @binds;
 
     # If we have foreign key relationship info, we need to join on those tables:
     if ($args->{foreign_keys}) {
@@ -1003,42 +1022,51 @@ SEARCHFORM
         }
     }
 
-    # If we have a query, we need to assemble a WHERE clause...
-    if (length $q) {
-        my ($column_data)
-            = grep { lc $_->{COLUMN_NAME} eq lc params->{searchfield} }
-            @{$columns};
-        debug(
-            "Searching on $column_data->{COLUMN_NAME} which is a "
-            . "$column_data->{TYPE_NAME}"
-        );
-        my $st = params->{searchtype};
-
-        if ($column_data) {
-            my $search_value = $q;
-            if ($st eq 'c' || $st eq 'nc') {
-                $search_value = '%' . $search_value . '%';
-            }
-
-            $query
-                .= " WHERE $table_name."
-                . $dbh->quote_identifier(params->{searchfield})
-                . ($st eq 'c' ? 'LIKE' :
-                   $st eq 'nc' ? 'NOT LIKE' :
-                   $st eq 'ne' ? '!=' : '=')
-                . $dbh->quote($search_value);
-
-            my $matchtype = $st eq "c" ? "contains": 
-                            $st eq "nc" ? "does not contain" :
-                            $st eq "ne" ? "does not equal": "equals";
-            $html
-                .= sprintf(
-                "<p>Showing results from searching for '%s' %s '%s'",
-                encode_entities(params->{searchfield}), $matchtype, encode_entities($q)
+    # If we have a query or a where_filter, we need to assemble a WHERE clause...
+    my $where_filter = $args->{where_filter};
+    if (length $q || $where_filter) {
+        my ($where_filter_sql, @where_filter_binds) = $dbh->generate_where_clauses($where_filter);
+        my (@search_wheres, @search_binds);
+        if (length $q) {    # this nested code is all for queries in $q
+            my ($column_data)
+                = grep { lc $_->{COLUMN_NAME} eq lc params->{searchfield} }
+                @{$columns};
+            debug(
+                "Searching on $column_data->{COLUMN_NAME} which is a "
+                . "$column_data->{TYPE_NAME}"
             );
-            $html .= sprintf '&mdash;<a href="%s">Reset search</a></p>',
-                _external_url($args->{dancer_prefix}, $args->{prefix});
+            my $st = params->{searchtype};
+
+            if ($column_data) {
+                my $search_value = $q;
+                if ($st eq 'c' || $st eq 'nc') {
+                    $search_value = '%' . $search_value . '%';
+                }
+
+                push(@search_wheres,
+                    "$table_name."
+                    . $dbh->quote_identifier(params->{searchfield})
+                    . ($st eq 'c' ? ' LIKE ' :
+                       $st eq 'nc' ? ' NOT LIKE ' :
+                       $st eq 'ne' ? ' != ' : ' = ')
+                   . '?' );
+                push(@search_binds, $search_value);
+
+                my $matchtype = $st eq "c" ? "contains": 
+                                $st eq "nc" ? "does not contain" :
+                                $st eq "ne" ? "does not equal": "equals";
+                $html
+                    .= sprintf(
+                    "<p>Showing results from searching for '%s' %s '%s'",
+                    encode_entities(params->{searchfield}), $matchtype, encode_entities($q)
+                );
+                $html .= sprintf '&mdash;<a href="%s">Reset search</a></p>',
+                    _external_url($args->{dancer_prefix}, $args->{prefix});
+            }
         }
+        # add the 'where' clauses to $query and the binds to @binds
+        $query .= " where " . join( " AND ", grep { length $_ } ($where_filter_sql, @search_wheres));
+        push(@binds, @where_filter_binds, @search_binds);
     }
 
     if ($args->{downloadable}) {
@@ -1150,7 +1178,7 @@ SEARCHFORM
 
     debug("Running query: $query");
     my $sth = $dbh->prepare($query);
-    $sth->execute()
+    $sth->execute(@binds)
         or die "Failed to query for records in $table_name - "
         . $dbh->errstr;
 
@@ -1568,7 +1596,7 @@ L<http://search.cpan.org/dist/Dancer-Plugin-SimpleCRUD/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-12 David Precious.
+Copyright 2010-16 David Precious.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
